/*
 * noanti
 *
 * [Misc]
 * NoAntiRadius=0
 *
 * Takes antiwarp away from any player with it activated in the
 *  center of the map.
 *
 * 23/11/03 - Created by Smong
 *  Warn message copied from MERVBot
 *
 * 14/02/04 - Decreased sample rate to 3x SendPositionDelay.
 *  Changed some stuff around.
 *
 */

#include "asss.h"
#include "util.h"

local void ArenaAction(Arena *arena, int action);
local int  timer(void *arena_);

enum { s_off, s_on };
local int noantikey;
#define NOANTI(a) (*(char*)P_ARENA_DATA(a, noantikey))

local Imodman     *mm;
local Iconfig     *cfg;
local Iplayerdata *pd;
local Igame       *game;
local Ichat       *chat;
local Iarenaman   *aman;
local Imainloop   *mainloop;

local void ArenaAction(Arena *arena, int action)
{
	int radius = cfg->GetInt(arena->cfg, "Misc", "NoAntiRadius", 0);

	if (action == AA_CREATE)
	{
		if (radius == 0)
		{
			NOANTI(arena) = s_off;
			return;
    		}
    	
		mainloop->SetTimer(timer, 500, cfg->GetInt(arena->cfg,
			"Misc", "SendPositionDelay", 20) * 3, arena, arena);
  	  	NOANTI(arena) = s_on;
  	}
	else if (action == AA_CONFCHANGED)
	{
		if (NOANTI(arena) == s_on)
		{
			mainloop->ClearTimer(timer, arena);
			NOANTI(arena) = s_off;
		}
		
		if (radius == 0) return;
		
		mainloop->SetTimer(timer, 500, cfg->GetInt(arena->cfg,
			"Misc", "SendPositionDelay", 20) * 3, arena, arena);
  	  	NOANTI(arena) = s_on;
	}
	else if (action == AA_DESTROY)
	{
		mainloop->ClearTimer(timer, arena);
		NOANTI(arena) = s_off;
	}
}

local int timer(void *arena_)
{
	Arena *arena = arena_;
	int x, y, r, radius, cost;
//	Link wlink;
//	LinkedList lst = { &wlink, &wlink };
	Target tgt;

	Link *link;
	Player *p;
	
	radius = cfg->GetInt(arena->cfg, "Misc", "NoAntiRadius", 0);
	radius = radius * radius;
	cost = cfg->GetInt(arena->cfg, "Cost", "AntiWarp", 0);

	tgt.type = T_PLAYER;

	pd->Lock();
	FOR_EACH_PLAYER(p)
	{
		if (p->arena != arena) continue;
		if (p->p_ship == SPEC) continue;
		if ( !IS_STANDARD(p) ) continue;
		if (p->status != S_PLAYING) continue;

		//Ignore players in safe
		if ( !(p->position.status & 0x08) ) continue;

		x = (p->position.x / 16) - 512;
		y = (p->position.y / 16) - 512;
		r = x*x + y*y;
		if (r > radius) continue;
/*		
		wlink.data = p;
		chat->SendAnyMessage(&lst, MSG_SYSOPWARNING, SOUND_BEEP1,
				"Antiwarp is not allowed in center. Type ?buy "
				"AntiWarp to regain antiwarp at cost of %d pts.",
				cost);
*/
		chat->SendSoundMessage(p, SOUND_BEEP1, "Antiwarp is not "
			"allowed in center. Type ?buy AntiWarp to regain "
			"antiwarp at cost of %d pts.", cost);

		tgt.u.p = p;
		game->GivePrize(&tgt, - PRIZE_ANTIWARP, 1);
	}
	pd->Unlock();

	return 1;
}

EXPORT const char info_noanti[] = "v1.1 <bz>";

EXPORT int MM_noanti(int action, Imodman *mm_, Arena *arena)
{
	if (action == MM_LOAD)
	{
		mm = mm_;
		cfg      = mm->GetInterface(I_CONFIG,     ALLARENAS);
		pd       = mm->GetInterface(I_PLAYERDATA, ALLARENAS);
		game     = mm->GetInterface(I_GAME,       ALLARENAS);
		chat     = mm->GetInterface(I_CHAT,       ALLARENAS);
		aman     = mm->GetInterface(I_ARENAMAN,   ALLARENAS);
		mainloop = mm->GetInterface(I_MAINLOOP,   ALLARENAS);
		if (!cfg || !pd || !game || !chat || !aman || !mainloop)
			return MM_FAIL;
		
		noantikey = aman->AllocateArenaData( sizeof(char) );
		if (noantikey == -1) return MM_FAIL;
		
		mm->RegCallback(CB_ARENAACTION, ArenaAction, ALLARENAS);
		
		return MM_OK;
	}
	else if (action == MM_UNLOAD)
	{
		mm->UnregCallback(CB_ARENAACTION, ArenaAction, ALLARENAS);
	
		aman->FreeArenaData(noantikey);

		mm->ReleaseInterface(cfg);
		mm->ReleaseInterface(pd);
		mm->ReleaseInterface(game);
		mm->ReleaseInterface(chat);
		mm->ReleaseInterface(aman);
		mm->ReleaseInterface(mainloop);
		return MM_OK;
	}

	return MM_FAIL;
}

